@@ -82,7 +82,7 @@ getDM<-function(data,DM,dist,nbStates,parNames,bounds,Par,cons,workcons,zeroInfl
         if(length(meanind)){
           sdind<-which(!apply(DM[[i]][1:nbStates+nbStates,,drop=FALSE],2,function(x) all(x==0)))
           newDM <- matrix(0,nrow(DM[[i]]),ncol=length(meanind)*2+length(sdind))
-          if(any(grepl("angleStrength",DM[[i]][,sdind]))) stop("angleStrength function only applies to angle mean")
+          if(any(grepl("angleFormula",DM[[i]][,sdind]))) stop("angleFormula function only applies to angle mean")
           newDM[,length(meanind)*2+1:length(sdind)]<-DM[[i]][,sdind]
           colnames(newDM)[length(meanind)*2+1:length(sdind)] <- DMnames[sdind]
           colnames(newDM)[seq(1,length(meanind)*2-1,2)] <- DMnames[meanind]
@@ -106,7 +106,7 @@ getDM<-function(data,DM,dist,nbStates,parNames,bounds,Par,cons,workcons,zeroInfl
           newDM <- DM[[i]]
         }
       }
-      if(!length(meanind) & any(grepl("angleStrength",DMterms))) stop("angleStrength function only applies to circular-circular regression model for angle mean")
+      if(!length(meanind) & any(grepl("angleFormula",DMterms))) stop("angleFormula function only applies to circular-circular regression model for angle mean")
       factorterms<-names(data)[unlist(lapply(data,is.factor))]
       factorcovs<-paste0(rep(factorterms,times=unlist(lapply(data[factorterms],nlevels))),unlist(lapply(data[factorterms],levels)))
       covs<-numeric()
