@@ -14,6 +14,7 @@
 #' distributions ('vm' and 'wrpcauchy').
 #' @param circularAngleMean Named list indicating whether to use circular-linear (FALSE) or circular-circular (TRUE) 
 #' regression on the mean of circular distributions ('vm' and 'wrpcauchy') for turning angles.  
+#' @param consensus Named list indicating whether to use the circular-circular regression consensus model
 #' @param stationary \code{FALSE} if there are covariates. If TRUE, the initial distribution is considered
 #' equal to the stationary distribution. Default: \code{FALSE}.
 #' @param cons Named list of vectors specifying a power to raise parameters corresponding to each column of the design matrix 
@@ -29,6 +30,7 @@
 #' @param nc indicator for zeros in fullDM
 #' @param meanind index for circular-circular regression mean angles with at least one non-zero entry in fullDM
 #' @param covsDelta data frame containing the delta model covariates (if any)
+#' @param workBounds named list of 2-column matrices specifying bounds on the working scale of the probability distribution, transition probability, and initial distribution parameters
 #' 
 #' @return A list of:
 #' \item{...}{Matrices containing the natural parameters for each data stream (e.g., 'step', 'angle', etc.)}
@@ -52,16 +54,18 @@
 #' 
 #' #natural parameter
 #' p <-   momentuHMM:::w2n(wpar,bounds,parSize,nbStates,nbCovs,m$conditions$estAngleMean,
-#' m$conditions$circularAngleMean,m$conditions$stationary,m$conditions$cons,m$conditions$fullDM,
+#' m$conditions$circularAngleMean,lapply(m$conditions$dist,function(x) x=="vmConsensus"),
+#' m$conditions$stationary,m$conditions$cons,m$conditions$fullDM,
 #' m$conditions$DMind,m$conditions$workcons,1,m$conditions$dist,m$conditions$Bndind,
-#' matrix(1,nrow=length(unique(m$data$ID)),ncol=1),covsDelta=m$covsDelta)
+#' matrix(1,nrow=length(unique(m$data$ID)),ncol=1),covsDelta=m$covsDelta,
+#' workBounds=m$conditions$workBounds)
 #' }
 #'
 #'
 #' @importFrom boot inv.logit
 #' @importFrom Brobdingnag as.brob sum
 
-w2n <- function(wpar,bounds,parSize,nbStates,nbCovs,estAngleMean,circularAngleMean,stationary,cons,fullDM,DMind,workcons,nbObs,dist,Bndind,nc,meanind,covsDelta)
+w2n <- function(wpar,bounds,parSize,nbStates,nbCovs,estAngleMean,circularAngleMean,consensus,stationary,cons,fullDM,DMind,workcons,nbObs,dist,Bndind,nc,meanind,covsDelta,workBounds)
 {
 
   # identify initial distribution parameters
@@ -69,7 +73,10 @@ w2n <- function(wpar,bounds,parSize,nbStates,nbCovs,estAngleMean,circularAngleMe
     nbCovsDelta <- ncol(covsDelta)-1 # substract intercept column
     
     foo <- length(wpar)-(nbCovsDelta+1)*(nbStates-1)+1
-    delta <- c(rep(0,nbCovsDelta+1),wpar[foo:length(wpar)])
+    
+    tmpwpar <- w2wn(wpar[foo:length(wpar)],workBounds$delta)
+    
+    delta <- c(rep(0,nbCovsDelta+1),tmpwpar)
     deltaXB <- covsDelta%*%matrix(delta,nrow=nbCovsDelta+1)
     expdelta <- exp(deltaXB)
     delta <- expdelta/rowSums(expdelta)
@@ -84,20 +91,29 @@ w2n <- function(wpar,bounds,parSize,nbStates,nbCovs,estAngleMean,circularAngleMe
   # identify regression coefficients for the transition probabilities
   if(nbStates>1) {
     foo <- length(wpar)-(nbCovs+1)*nbStates*(nbStates-1)+1
-    beta <- wpar[foo:length(wpar)]
+    
+    tmpwpar <- w2wn(wpar[foo:length(wpar)],workBounds$beta)
+    
+    beta <- tmpwpar
     beta <- matrix(beta,nrow=nbCovs+1)
     wpar <- wpar[-(foo:length(wpar))]
   }
   else beta <- NULL
   
   distnames <- names(dist)
-  parindex <- c(0,cumsum(unlist(lapply(fullDM,ncol)))[-length(fullDM)])
+  parCount<- lapply(fullDM,ncol)
+  for(i in distnames[unlist(circularAngleMean)]){
+    parCount[[i]] <- length(unique(gsub("cos","",gsub("sin","",colnames(fullDM[[i]])))))
+  }
+  parindex <- c(0,cumsum(unlist(parCount))[-length(fullDM)])
   names(parindex) <- names(fullDM)
 
   parlist<-list()
   
   for(i in distnames){
-    tmpwpar<-wpar[parindex[[i]]+1:ncol(fullDM[[i]])]
+    
+    tmpwpar<-wpar[parindex[[i]]+1:parCount[[i]]]
+    
     if(estAngleMean[[i]] & Bndind[[i]]){ 
       bounds[[i]][,1] <- -Inf
       bounds[[i]][which(bounds[[i]][,2]!=1),2] <- Inf
@@ -110,7 +126,7 @@ w2n <- function(wpar,bounds,parSize,nbStates,nbCovs,estAngleMean,circularAngleMe
       tmpwpar[(foo - nbStates):(foo - 1)] <- angleMean
       tmpwpar[foo:length(tmpwpar)] <- kappa
     }
-    parlist[[i]]<-w2nDM(tmpwpar,bounds[[i]],fullDM[[i]],DMind[[i]],cons[[i]],workcons[[i]],nbObs,circularAngleMean[[i]],nbStates,0,nc[[i]],meanind[[i]])
+    parlist[[i]]<-w2nDM(tmpwpar,bounds[[i]],fullDM[[i]],DMind[[i]],cons[[i]],workcons[[i]],nbObs,circularAngleMean[[i]],consensus[[i]],nbStates,0,nc[[i]],meanind[[i]],workBounds[[i]])
 
     if((dist[[i]] %in% angledists) & !estAngleMean[[i]]){
       tmp<-matrix(0,nrow=(parSize[[i]]+1)*nbStates,ncol=nbObs)
@@ -126,8 +142,24 @@ w2n <- function(wpar,bounds,parSize,nbStates,nbCovs,estAngleMean,circularAngleMe
   return(parlist)
 }
 
-w2nDM<-function(wpar,bounds,DM,DMind,cons,workcons,nbObs,circularAngleMean,nbStates,k=0,nc,meanind){
+w2wn <- function(wpar,workBounds,k=0){
   
+  ind1<-which(is.finite(workBounds[,1]) & is.infinite(workBounds[,2]))
+  ind2<-which(is.finite(workBounds[,1]) & is.finite(workBounds[,2]))
+  ind3<-which(is.infinite(workBounds[,1]) & is.finite(workBounds[,2]))
+  
+  wpar[ind1] <- exp(wpar[ind1])+workBounds[ind1,1]
+  wpar[ind2] <- (workBounds[ind2,2]-workBounds[ind2,1]) * boot::inv.logit(wpar[ind2])+workBounds[ind2,1]
+  wpar[ind3] <- -(exp(-wpar[ind3]) - workBounds[ind3,2])
+  
+  if(k) wpar <- wpar[k]
+  return(wpar)
+}
+
+w2nDM<-function(wpar,bounds,DM,DMind,cons,workcons,nbObs,circularAngleMean,consensus,nbStates,k=0,nc,meanind,workBounds){
+  
+  wpar <- w2wn(wpar,workBounds)
+
   a<-bounds[,1]
   b<-bounds[,2]
   
@@ -140,7 +172,14 @@ w2nDM<-function(wpar,bounds,DM,DMind,cons,workcons,nbObs,circularAngleMean,nbSta
   ind2<-which(zoInd)
   ind3<-which(!piInd & !zoInd)
   
-  XB <- p <- getXB(DM,nbObs,wpar,cons,workcons,DMind,circularAngleMean,nbStates,nc,meanind)
+  if(!consensus){
+    XB <- p <- getXB(DM,nbObs,wpar,cons,workcons,DMind,circularAngleMean,consensus,nbStates,nc,meanind)
+    l_t <- matrix(1,nrow(XB),ncol(XB))
+  } else {
+    tmpXB <- getXB(DM,nbObs,wpar,cons,workcons,DMind,circularAngleMean,consensus,nbStates,nc,meanind)
+    XB <- p <- tmpXB$XB
+    l_t <- matrix(tmpXB$l_t,nrow(XB),ncol(XB))
+  }
   
   if(length(ind1) & !circularAngleMean)
     p[ind1,] <- (2*atan(XB[ind1,]))
@@ -163,12 +202,15 @@ w2nDM<-function(wpar,bounds,DM,DMind,cons,workcons,nbObs,circularAngleMean,nbSta
   ind32<-ind3[which(is.finite(a[ind3]) & is.finite(b[ind3]))]
   ind33<-ind3[which(is.infinite(a[ind3]) & is.finite(b[ind3]))]
   
-  p[ind31,] <- (exp(XB[ind31,,drop=FALSE])+a[ind31])
-  p[ind32,] <- ((b[ind32]-a[ind32])*boot::inv.logit(XB[ind32,,drop=FALSE])+a[ind32])
+  p[ind31,] <- (l_t[ind31,,drop=FALSE] * exp(XB[ind31,,drop=FALSE])+a[ind31])
+  p[ind32,] <- ((b[ind32]-a[ind32])*(l_t[ind32,,drop=FALSE] * boot::inv.logit(XB[ind32,,drop=FALSE]))+a[ind32])
   p[ind33,] <- -(exp(-XB[ind33,,drop=FALSE]) - b[ind33])
   
-  if(any(p<a | p>b))
-    stop("Scaling error. Check initial values and bounds.")
+  if(!any(is.na(p))){ 
+    if(any(p<a | p>b)){
+      stop("Scaling error. Check initial values and bounds.")
+    }
+  }
   
   if(k) {
     p <- p[k]
